/**************************************************************************//**
 * @file
 * @brief Empty Project
 * @author Energy Micro AS
 * @version 3.20.2
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2014 Silicon Labs, http://www.silabs.com</b>
 *******************************************************************************
 *
 * This file is licensed under the Silicon Labs Software License Agreement. See 
 * "http://developer.silabs.com/legal/version/v11/Silicon_Labs_Software_License_Agreement.txt"  
 * for details. Before using this software for any purpose, you must agree to the 
 * terms of that agreement.
 *
 ******************************************************************************/
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "em_device.h"
#include "em_chip.h"
#include "em_emu.h"
#include "em_cmu.h"
#include "em_gpio.h"
#include "em_usart.h"
#include "bsp.h"
#include "retargetvirtual.h"

#include "visi_genie.h"
#include "rtcdriver.h"

/* Function prototypes */
void uartSetup(void);
void cmuSetup(void);

/* Setup UART1 in async mode for RS232*/
static USART_InitAsync_TypeDef uartInit = USART_INITASYNC_DEFAULT;

bool winButton0Status = false; //holds the "status" of winButton0 object.
bool winButton1Status = false; //holds the "status" of winButton1 object.

int userLed0Status = 0;
int userLed1Status = 0;

void myGenieEventHandler(void)  //Event handler for the 4d Systems display
{
    union genieFrame Event;
    genieDequeueEvent(&Event);

    //event report from an object
    if(Event.reportObject.cmd == GENIE_REPORT_EVENT)
    {
        if (Event.reportObject.object == GENIE_OBJ_WINBUTTON) // If the Reported Message was from a winbutton
        {
            if (Event.reportObject.index == 0)
            {
                if (winButton0Status == false) {
                    printf("Button 1 pressed\n\r");
                    printf("LED 1 is On now\n\r");
                    BSP_LedSet(0);
                    userLed0Status = 1;
                    winButton0Status = !winButton0Status;
                }

                 else if (winButton0Status == true) {
                    printf("Button 1 pressed\n\r");
                    printf("LED 1 is Off now\n\r");
                    BSP_LedClear(0);
                    userLed0Status = 0;
                    winButton0Status = !winButton0Status;
                }
            }

            if (Event.reportObject.index == 1)
            {
                if(winButton1Status == false) {
                    printf("Button 2 pressed\n\r");
                    printf("LED 2 is On now\n\r");
                    BSP_LedSet(1);
                    userLed1Status = 1;
                    winButton1Status = !winButton1Status;
                }

                else if(winButton1Status == true) {
                    printf("Button 2 pressed\n\r");
                    printf("LED 2 is Off now\n\r");
                    BSP_LedClear(1);
                    userLed1Status = 0;
                    winButton1Status = !winButton1Status;
                }

            }

        }
    }
}

/**************************************************************************//**
 * @brief  Main function
 *****************************************************************************/
int main(void)
{
  /* Initialize chip - handle erratas */
  CHIP_Init( );

  // Initialization of RTCDRV driver
  RTCDRV_Init();

  /* Initialize clocks and oscillators */
  cmuSetup( );

  /* Initialize UART peripheral */
  uartSetup( );

  /* Initialize LED driver */
  BSP_LedsInit();

  /* Initialize USART and map LF to CRLF */
  RETARGET_SerialInit();
  RETARGET_SerialCrLf(1);

  printf("Shakeeb's SiLabs' Gecko board Winbutton-LED Demo for uLCD43-DCT \n\r");  // Display a welcome message on the serial monitor

  /* Start 4D display */
  Start();
  genieAttachEventHandler(&myGenieEventHandler); // Call the event handler


  /* Infinite loop */
  while (1)
  {
	    genieWriteObject(GENIE_OBJ_USER_LED,0,userLed0Status);      // Set the USERLED0 object high or low depending on state of button 0
	    genieWriteObject(GENIE_OBJ_USER_LED,1,userLed1Status);      // Set the USERLED1 object high or low depending on state of button 1
  }
}

/***************************************************************************//**
 * @brief Set up Clock Management Unit
 ******************************************************************************/
void cmuSetup(void)
{
  /* Start HFXO and wait until it is stable */
  /* CMU_OscillatorEnable( cmuOsc_HFXO, true, true); */

  /* Select HFXO as clock source for HFCLK */
  /* CMU_ClockSelectSet(cmuClock_HF, cmuSelect_HFXO ); */

  /* Disable HFRCO */
  /* CMU_OscillatorEnable( cmuOsc_HFRCO, false, false ); */

  /* Enable clock for HF peripherals */
  CMU_ClockEnable(cmuClock_HFPER, true);

  /* Enable clock for USART module */
  CMU_ClockEnable(cmuClock_UART1, true);
}



/******************************************************************************
* @brief  uartSetup function
*
******************************************************************************/
void uartSetup(void)
{
  /* Enable clock for GPIO module (required for pin configuration) */
  CMU_ClockEnable(cmuClock_GPIO, true);
  /* Configure GPIO pins */
  GPIO_PinModeSet(gpioPortB, 9, gpioModePushPull, 1);
  GPIO_PinModeSet(gpioPortB, 10, gpioModeInput, 0);

  /* Reset Pin*/
  GPIO_PinModeSet(gpioPortB, 11, gpioModeInput, 1);

  /* Prepare struct for initializing UART in asynchronous mode*/
  uartInit.enable       = usartDisable;   /* Don't enable UART upon initialization */
  uartInit.refFreq      = 0;              /* Provide information on reference frequency. When set to 0, the reference frequency is */
  uartInit.baudrate     = 115200;         /* Baud rate */
  uartInit.oversampling = usartOVS16;     /* Over sampling. Range is 4x, 6x, 8x or 16x */
  uartInit.databits     = usartDatabits8; /* Number of data bits. Range is 4 to 10 */
  uartInit.parity       = usartNoParity;  /* Parity mode */
  uartInit.stopbits     = usartStopbits1; /* Number of stop bits. Range is 0 to 2 */
  uartInit.mvdis        = false;          /* Disable majority voting */
  uartInit.prsRxEnable  = false;          /* Enable USART Rx via Peripheral Reflex System */
  uartInit.prsRxCh      = usartPrsRxCh0;  /* Select PRS channel if enabled */

  /* Initialize USART with uartInit struct */
  USART_InitAsync(uart, &uartInit);

  /* Prepare UART Rx and Tx interrupts */
  USART_IntClear(uart, _UART_IFC_MASK);
  USART_IntEnable(uart, UART_IEN_RXDATAV);
  NVIC_ClearPendingIRQ(UART1_RX_IRQn);
  NVIC_ClearPendingIRQ(UART1_TX_IRQn);
  NVIC_EnableIRQ(UART1_RX_IRQn);
  NVIC_EnableIRQ(UART1_TX_IRQn);

  /* Enable I/O pins at UART1 location #2 */
  uart->ROUTE = UART_ROUTE_RXPEN | UART_ROUTE_TXPEN | UART_ROUTE_LOCATION_LOC2;

  /* Enable UART */
  USART_Enable(uart, usartEnable);
}


